#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-rgma-client-config v. 5.2.2
#
# Post-installation script for configuring the gLite RGMA Service Publisher Service
# Joachim Flammer <Joachim.Flammer@cern.ch>
#
# Version info: $Id: glite-rgma-client-config.py,v 1.28.2.6 2006/05/02 10:26:57 flammer Exp $
#
# Usage: python glite-rgma-client-config [-c|-v|-h|--help]
#        -c                  print configuration
#        -v                  print version
#        -h,--help           print usage info
#        --verbose           turn verbosity on
#
#        --configure         configure client
#        --siteconfig=URL    specify URL of the site configuration file
#        --subservice        run as subservice script
#
# Return codes: 0 - Ok
#               1 - Failed
#
#####################################################################################

import os,string,socket,pwd
import sys
import getopt
import commands

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams

# Set global variables here 
global params                         # all config values from the XML file

class glite_rgma_client:

    #################################################################
    # init 
    #
    # init
    #
    #################################################################        
    def __init__(self):
        self.verbose = 0
        self.version = "5.2.2"
        self.name = "glite-rgma-client"
        self.friendly_name = "gLite R-GMA Client"
        

    #################################################################
    # banner 
    #
    # print banner
    #
    #################################################################    
    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "            Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #################################################################
    # copyright 
    #
    # print copyright
    #
    #################################################################    
    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'


    #################################################################
    # version
    #
    # print version
    #
    #################################################################    
    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    

    #################################################################
    # usage 
    #
    # print usage
    #
    #################################################################    
    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()

        print "Usage: \n"
        print "Edit the configuration file %s.cfg.xml in " %  self.name,
        print "%s/etc.config/templates" % os.environ['GLITE_LOCATION']
        print "save it as %s/etc/config/%s.cfg.xml" % (os.environ['GLITE_LOCATION'], self.name),
        print "and run the script as follows"
        print "python %s-config [OPTION...]" % self.name
        print ''
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --verbose           turn verbosity on'
        print ''
        print '    --configure         configure %s' % self.friendly_name
        print '    --subservice        run script as a subservice'
        print ''
        print '    --siteconfig=URL    URL of the site configuration file'
        print ''
        print '\n'
    

    #################################################################
    # configure
    #
    # configure rgma client
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):

        # ------------------------------------------------------------
        # get corresponding configuration values
        # ------------------------------------------------------------
        
        errorFlag = 0
        
        print "Checking configuration values                             ",        

        # -- internal --
        
        # subservice                    
        if params.has_key('rgma.client.run_as_subservice'):
            subservice = params['rgma.client.run_as_subservice']
        else:
            subservice = "false"
        
        # -- R-GMA common --
                                        
        # rgma server hostname (user defined)
        if params.has_key('rgma.server.hostname'):
            server_hostname = params['rgma.server.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.server.hostname\" not found.")
            return 1                
        
        # rgma schema hostname (user defined)
        if params.has_key('rgma.schema.hostname'):
            schema_hostname = params['rgma.schema.hostname']
        else:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.schema.hostname\" not found.")
            return 1

        # rgma registry host: at least one must be given (user defined)
        if not len(params['rgma.registry.hostname']) > 0:
            glib.printFailedMessage()
            glib.printErrorMessage("Parameter \"rgma.registry.hostname\" not found.")
            return 1
                
        # rgma secure mode (advanced)
        if params.has_key('rgma.secure.mode'):
            rgma_secure_mode = params['rgma.secure.mode']
        
        # -- global parameters --
         
        # host certificate file (advanced)          
        if params.has_key('host.certificate.file'):
            host_certificate_file = params['host.certificate.file']
            
        # host.key.file (advanced)
        if params.has_key('host.key.file'):
            host_key_file = params['host.key.file']
            
        # ca.certificates.dir (advanced)
        if params.has_key('ca.certificates.dir'):
            ca_certificates_dir = params['ca.certificates.dir']
            
        # user certificates path (advanced)
        if params.has_key('user.certificate.path'):
            user_certificates_path = params['user.certificate.path']
                               
        glib.printOkMessage()                    

        # temporary message as long as only one registry is allowed
        if len(params['rgma.registry.hostname']) > 1:
            glib.printWarningMessage("You specified more than one registry.")
            glib.printWarningMessage("At the moment only one registry is allowed.")
            glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
            glib.printWarningMessage("and ignore the others.")

        # ------------------------------------------------------------        
        # Check certificates
        # ------------------------------------------------------------    
        if subservice=="false":
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s service" \
                            % service.friendly_name
                        sys.exit(1) 
    
        # ------------------------------------------------------------
        # Run security util script
        # ------------------------------------------------------------
                
        if subservice=="false":
            print "Configuring security ...                                 "
            errorcode = os.system("%s/etc/config/scripts/glite-security-utils-config.py --subservice" % os.environ['GLITE_LOCATION'])
    
            if errorcode:
                print "Configuring security                                      ",
                glib.printFailedMessage()
                return 1
        
            print "Configuring security                                      ",
            glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Verify the $GLITE_LOCATION/etc directory structure
        #-------------------------------------------------------------------------------
        
        print "Verifying the %s/etc directory structure          " % os.environ['GLITE_LOCATION'],
    
        path = '%s/etc' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
            glib.printFailedMessage()
            glib.printErrorMessage('%s/etc does not exist! Please verify your gLite installation' % os.environ['GLITE_LOCATION'])
            return 1                        
        
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #-------------------------------------------------------------------------------

        if glib.checkJava() != 0:
            return 1        
                                    
        # ------------------------------------------------------------
        # configure R-GMA
        # ------------------------------------------------------------

        registries_string = ""

# the following line should be put back as soon as multiple registries are enabled again            
####
#        for item in params['rgma.registry.hostname']:
#
#            registries_string = registries_string + " --registry %s" % item                        
#            if self.verbose:
#                print "Setting up R-GMA to use registry server '%s'" % item
####

        # the following lines should be removed as soon as multiple registries are enabled again
        # start-new-codeblock
        registries_string = "--registry %s" % params['rgma.registry.hostname'][0]
        if self.verbose:
            print "Setting up R-GMA to use registry server '%s'" % params['rgma.registry.hostname'][0]
        # end-new-codeblock
    
        print "Configuring R-GMA                                         ",        

#        errorcode =0
#        errorcode = glib.export('RGMA_HOME', '${GLITE_LOCATION}')
#
#        if errorcode:
#            glib.printFailedMessage()
#            glib.printErrorMessage("Could not export RGMA_HOME environment variable.")
#            return 1
#    
        # decide on security setting
        if rgma_secure_mode=="true":
            secure_setting="--secure=yes"
        else:
            secure_setting="--secure=no"
                
        # add server, schema and registry(ies)                                    
        errorcode = 0
        if self.verbose == 0:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s --silent'  \
                % (os.environ['GLITE_LOCATION'], server_hostname, schema_hostname, registries_string, secure_setting))
        else:
            errorcode += os.system('python %s/share/rgma/scripts/rgma-setup.py --server %s --schema %s %s %s'  \
                % (os.environ['GLITE_LOCATION'], server_hostname, schema_hostname, registries_string, secure_setting))
        
        if errorcode:
            glib.printFailedMessage()
            return 1

        glib.printOkMessage()            
    
        # ------------------------------------------------------------
        # configure R-GMA client security
        # ------------------------------------------------------------        
        
        print "Configuring %s security                   " % self.friendly_name,
        
        path = "%s/etc/rgma/ClientAuthentication.props" % os.environ['GLITE_LOCATION']

        try:
            file = open(path, 'w')
#            file.write('sslCertFile=%s/%s/%s\n' % (rgma_user_home_dir, user_certificates_path, user_cert_filename))
#            file.write('sslKey=%s/%s/%s\n' % (rgma_user_home_dir, user_certificates_path, user_key_filename))
            file.write('sslCertFile=\n')
            file.write('sslKey=\n')
            file.write('crlFiles=%s/*.r0\n' % ca_certificates_dir)
            file.write('sslCAFiles=%s/*.0\n' % ca_certificates_dir)
            file.write('#gridProxyFile=/tmp/grid-user-proxy.pem\n')
            file.close()
        except IOError, ex:
            glib.printFailedMessage()
            if ex.strerror == 'Permission denied':
                glib.printErrorMessage("File '%s' is not writable" % path)
                return 1
            else:
                glib.printErrorMessage(ex.strerror)
                return 1

        glib.printOkMessage()                    
            
        return 0
                

    #################################################################
    # showConfiguration
    #
    # show configuration
    #
    # return 0         ok 
    #        1         error
    #################################################################
    def showConfiguration(self):
        
        self.copyright()
        self.showVersion()
        glib.print_params(params)

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "             Main settings of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        print "  R-GMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.server.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.server.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.server.hostname' not found.")
        print "     --------------------------------------------------------------"
        print
        
        # print schema settings
        print "  R-GMA SCHEMA SERVER"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.schema.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.schema.hostname']
        else:
            glib.printErrorMessage("Parameter 'rgma.schema.hostname' not found.")        
        print "     --------------------------------------------------------------"
        print
        
        # print registry settings
        print "  R-GMA REGISTRY SERVER"        
        print "     --------------------------------------------------------------"
# The following lines should be put in place as soon as multiple registries in enabled again
######
#        if params.has_key('rgma.registry.hostname'):
#            for item in params['rgma.registry.hostname']: 
#                print "     \033[34m[Location]\x1b[0m          %s " % item
#        else:
#            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")        
######

        # The following lines can be removed as soon as multiple registries are enabled again
        # start-new-block
        if params.has_key('rgma.registry.hostname'):
            print "     \033[34m[Location]\x1b[0m          %s " % params['rgma.registry.hostname'][0]
        else:
            glib.printErrorMessage("Parameter 'rgma.registry.hostname' not found.")    
        
        if params.has_key('rgma.registry.hostname'):
            if len(params['rgma.registry.hostname']) > 1:
                print
                glib.printWarningMessage("You specified more than one registry.")
                glib.printWarningMessage("At the moment only one registry is allowed.")
                glib.printWarningMessage("Configuration will use first specified registry '%s'" % params['rgma.registry.hostname'][0])
                glib.printWarningMessage("and ignore the others.")
                print        
        # end-new-block

        print "     --------------------------------------------------------------"
        print
        
        print "  R-GMA Client"
        print "     --------------------------------------------------------------"
        if params.has_key('rgma.secure.mode'):
            print "     \033[34m[SecureMode]\x1b[0m        %s " % params['rgma.secure.mode']
        else:
            glib.printErrorMessage("Parameter 'rgma.secure.mode' not found.")   

        print "     --------------------------------------------------------------"

        return 0


#################################################################
# loadDefaultParameters
#
# load the default parameters
#
# return 0 ok
#        1 error
#################################################################
def loadDefaultParameters(params):
   
    # R-GMA client default parameters
    params['set.proxy.path'] = 'false'
    
    # R-GMA common default parameters
    params['rgma.user.name']   = "rgma"
    params['rgma.user.group']  = "rgma"
    params['rgma.secure.mode'] = "true"
    
    return 0


#################################################################
# set_env
#
# setup environment
# 
#################################################################
def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment file will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
#        if os.path.exists(params['GLITE_LOCATION']):
#            if not os.access(params['GLITE_LOCATION'], os.W_OK):
#                print "    GLITE_LOCATION is set to the non-writable directory %s -" % params['GLITE_LOCATION'],
#                params['GLITE_LOCATION'] = "%s" % (installPrefix)
#                print "it will be changed to %s" % params['GLITE_LOCATION']
#            else:
#                if not params['GLITE_LOCATION'] == installPrefix:
#                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
#                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
#                    sys.exit(1)
#        else:
        params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        if os.path.exists(params['ca.certificates.dir']):
            if not os.access('ca.certificates.dir', os.W_OK):
                print "    ca.certificates.dir is set to the non-writable directory %s - " % params['ca.certificates.dir'],
                params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
                print "it will be changed to %s" % params['ca.certificates.dir']
            else:
                if not params['ca.certificates.dir'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
        
        print "\n"

    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        glib.check_dir(os.environ['GLITE_LOCATION_TMP'])

    glib.export('CATALINA_HOME')
    glib.export('JAVA_HOME')
    
    if not params.has_key('set.proxy.path'):
        params['set.proxy.path'] = 'false'
    if params['set.proxy.path'] == "true":
        glib.export('GRID_PROXY_FILE', "/tmp/x509up_u`id -u`")
        glib.export('X509_USER_PROXY', "/tmp/x509up_u`id -u`")

    # bin and lib paths
    glib.addEnvPath("PATH","$GLITE_LOCATION/externals/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/bin")
    glib.addEnvPath("PATH","$JAVA_HOME/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")
    
    glib.addEnvPath("PYTHONPATH","$GLITE_LOCATION/lib/python")

    # Set environment
    glib.setUserEnv()

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':
    
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    verboseCLI = 0
    
    #-------------------------------------------------------------------------------    
    # Load parameters
    #-------------------------------------------------------------------------------    

    # Get an instance of the ConfigParams class
    params = ConfigParams()

    # set default parameters
    loadDefaultParameters(params)    
    
    try:
        opts, args = glib.getopt(sys.argv[1:],'chv', ['checkconf', 'help', 'version','siteconfig=', 'configure', 'verbose', 'subservice'])
        for o, a in opts:
                        
            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1
                
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    
    except getopt.GetoptError:
        pass    
    
    # load rgma common and rgma client configuration parameters
    configFiles = ["%s/../glite-rgma-client.cfg.xml" % glib.getScriptPath(), 
                   "%s/../glite-rgma-common.cfg.xml" % glib.getScriptPath()]
        
    if glib.loadConfiguration(configFiles,params):
        glib.printErrorMessage("Problems while loading configuration file.")
        sys.exit(1)
    
    # ------------------------------------------------------------    
    # set verbosity
    # ------------------------------------------------------------    
    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    
    glib.verbose = verbose
        
    # ------------------------------------------------------------    
    # Set up the environment
    # ------------------------------------------------------------    
    set_env()
          
    # ------------------------------------------------------------    
    # Instantiate the service classes
    # ------------------------------------------------------------    
    service = glite_rgma_client()
    service.verbose = verbose
    
    # ------------------------------------------------------------
    # Command line opts if any
    # ------------------------------------------------------------
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','siteconfig=', 'configure', 'verbose', 'subservice'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    # ------------------------------------------------------------
    # Check cli options
    # ------------------------------------------------------------
    configure  = 0
    subservice = 0
    
    for o, a in opts:
        # help
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
            
        # show version
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        
        # check configuration
        if o in ("-c", "--checkconf"):
            if service.showConfiguration():
                sys.exit(1)
            sys.exit(0)

        # configure service
        if o == "--configure":
            configure = 1

        # configure service
        if o == "--subservice":
            subservice = 1
            params['rgma.client.run_as_subservice'] = "true"

    #-------------------------------------------------------------------------------    
    # Configure
    #-------------------------------------------------------------------------------    
    if configure==1:
        
        #-------------------------------------------------------------------------------    
        # Print configuration parameters
        #-------------------------------------------------------------------------------    
        if verbose:
            glib.print_params(params)

        #-------------------------------------------------------------------------------
        # print default statements
        #-------------------------------------------------------------------------------    
        if subservice==0:
            service.copyright()
            service.showVersion()
       
        # print banner
        service.banner()
    
        #-------------------------------------------------------------------------------
        # Configure the R-GMA client
        #-------------------------------------------------------------------------------    
        if service.configure() != 0:
            glib.printErrorMessage("\nAn unrecoverable error occurred while configuring the %s service" % service.friendly_name)
            sys.exit(1)
        else:
            print "\n%s configuration successfully completed\n" % service.friendly_name            
            if subservice==0:
                print "To check if your client installation is working correctly you can run "
                print "   %s/bin/rgma-client-check" % os.environ['GLITE_LOCATION']
                print
                print "Please source "
                print "    %s" % params['installer.export.filename']
                print "to update your shell environment or logout and login for the"
                print "automatic update to take place"
                print                
            sys.exit(0)
            
    else:
        # ------------------------------------------------------------
        # No option given
        # ------------------------------------------------------------
        service.usage()
        sys.exit(0)            
    
